#
# shellscript, generate xfig3.2-picture from daten.def
#  by Joerg.Schulenburg
#
# use: def2fig.sh [-h] [-c|-C] [-s 1] [-z 1] [-i daten.def] [>out.fig]
#   option: -s factor  - scale sites by factor
#           -z factor  - scale factor (zoom)
#           -c         - move sites around the center of UC 
#           -C         - move sites inside of UC
#           -i file    - input file
#           -h         - this help (nothing else)
#
# v1.8  Apr09 - add zoom factor for small unitcells
# v1.7  Nov07 - adapt to new daten.def format
# v1.61 Jul07 - fix language specifique outputs for awk-printf-%f
# v1.6  Dec04 - fix error with dashed bonds
# v1.5  Mai04 - new options -c -i -s, now inputfile only as option, use stderr
# v1.4  Aug01 - 3d version
# v1.3  Feb01 - J1, J2 different (dashed)
# v1.2  Apr00 - circles around sites, better xfig-file
# v1.1  Sep99 - tail +n | head -1 replaced by awk
#
# TODO: use only bonds? (direction of bonds, easier to draw)
#
# the daten.def file should have additional lines for better drawing:
# #===base=== b1x b1y b2x b2y        - base vektors b1 b2 default=( 1.0 0.0 0.0 1.0 )
# #===edge=== e1x e1y e2x e2y        - set of edge-vectors used for PBC, units b1,b2
# #===pbcf=== x0 y0 x1 y1 .. x3 y3  - PBC-frame for the cluster in units of b1,b2 (best choice: 0 0 e1x e1y e1x+e2x e1y+e2y  e2x e2y)
#
LANG=C    # we want to have (printf "%2.2f",2) as 2.00 not 2,00!
scale=1   # scale factor
zoom=1    # zoom factor
center=0  # if set to 1 center sites for compact cluster
file=daten.def
echo "num_args=$#" >&2
while test $# -gt 0; do
 if test "$1" == "-s"; then scale=$2; shift; shift; continue; fi
 if test "$1" == "-z"; then zoom=$2;  shift; shift; continue; fi
 if test "$1" == "-c"; then center=1; shift; continue; fi
 if test "$1" == "-C"; then center=2; shift; continue; fi
 if test "$1" == "-i"; then file=$2;  shift; shift; continue; fi
 head -15 $0 >&2   # help
 exit
done
if test ! -r $file; then echo "file $file not found" >&2 ; exit; fi
AWK=awk
if which gawk >/dev/null; then AWK=gawk; fi   # use better awk
#
# see /usr/doc/info/gawk.info.gz
#
# XFIG 3.2: 1200dpi (new)
#   /usr/share/doc/packages/xfig/FORMAT3.2
#   circle:1 sub={3=circle_by_R} lstyle=0 width=1 pcolor=0  fcolor=7  depth=9\
#          unused=0 afill=20 dashlen=0.000 direction=1 angle=0.000\
#          cx=6030 cy=1170 rx=229 ry=229 x1=6030 y1=1170 x2=6259 y2=1170\n
#   pline:2 sub={1=polyline} style={0,1=dashed} width=2 pcolor=-1 fcolor=-1\
#          depth=0 unused=0 afill=-1 dashlen=0.000 join=0 cap=0 radius=-1\
#          farrow=0 barrow=0 npoints=2\n\t x1 y1 x2 y2\n
#   string:4 justifyLCR=0 color=-1 depth=0 unused=0 font=0 fsize=18 angle=0.0\
#          fflags={bitvector:4} dy=195 dx=n*135
#            x1 y1=lower_base string\001\n
# print xfig-header
printf "#FIG 3.2\nLandscape\nCenter\nMetric\nA4\n100.0\nSingle\n-2\n1200 2\n"
# grep min/max coordinates for shift and scaling of the produced picture
xx=$( $AWK '\
  BEGIN { count=0;nr=0;ox=0;oy=0;oz=0;rx=0;ry=0;rz=0 }
  !/^\#/ { count++; nr=count; } /^\#/ { nr=0; } (nr==1) { vn=$1;vnw=$2; }
  (nr > 1) && (nr < vn+2) {\
       if(($2)<ox) ox=($2); if(($2)>rx) rx=($2); 
       if(($3)<oy) oy=($3); if(($3)>ry) ry=($3);
       if(($4)<oz) oz=($4); if(($4)>rz) rz=($4);
     }
  END { printf " %f %f %f %f %f %f\n",ox,rx-ox,oy,ry-oy,oz,rz-oz }
  ' $file )
echo "# box=$xx (ox,rx,oy,ry,oz,rz)"
# ToDo: compute auto zoom
ox=$( echo "$xx" | $AWK '{ print -($1)+1.2 }' )
oy=$( echo "$xx" | $AWK '{ print $4+1 }' )
oz=$( echo "$xx" | $AWK '{ print $5 }' )
rz=$( echo "$xx" | $AWK '{ print $6 }' )
# create the xfig-picture
# awk-functions only work with gawk, not tru64-awk, ToDo: check it!
$AWK -v ox=$ox -v oy=$oy -v oz=$oz -v rz=$rz\
 -v scale=$scale -v zoom=$zoom -v center=$center '
  function outs( x, y, z, s, c){ # number  c=color s=number  draw a site
    printf "# site (xyz,num,color) %f %f %f %d %d\n", x, y, z, s, c;
    printf "1 3 0 1 0 7 4 0 20 0.00 1 0.00 %d %d %d %d %d %d %d %d\n",
        (x+ox)*sc,(y+oy)*sc,230*scale,230*scale,
        (x+ox)*sc,(y+oy)*sc,(x+ox)*sc+230*scale,(y+oy)*sc;
    printf "4 1 %d 0 0 0 %d 0.00000 4 %d %d %d %d %d\\001\n",
        c,20*scale,210,150,(x+ox)*sc,(y+oy)*sc+100*scale,s }
  function outc( x, y, z, s, c){ # string  c=color s=string
    printf "4 0 %d 0 0 0 18 0.00000 4 195 135 %d %d %s\\001\n",c,(x+ox)*sc+10,(y+oy)*sc+7,s }
  BEGIN { count=0; nr=0; sc=80*15*zoom; d=2;
          for( i=0; i<3; i++ ) for( j=0; j<3; j++ ) { b[i*3+j]=0; e[i*3+j]=0 }
          for( i=0; i<3; i++ ) { b[i*3+i]=1; e[i*3+i]=1; }
          b[1*3+1]=-1; # we invert e_y=down of xfig
          c1x=rx/2; c1y=ry/2; # center of BZ for compact figure
        }
  !/^\#/ { count++; nr=count; }
  /^\#/  { print; nr=0 ; }
  /^\#===base===/  {   # usually its (1,0,0),(0,1,0),(0,0,1) or (1,0),(0,1)
    if ( NF<10 )   { d=2 } else { d=3 }
    for( i=0; i<d; i++ ) for( j=0; j<d; j++ ) b[i*3+j]=$(2+i*d+j);
    for( i=0; i<d; i++ ) b[i*3+1]=-b[i*3+1];  # we invert e_y=down of xfig
    print "base-vector ",b[0],b[1],b[3],b[4] > "/dev/stderr"
  }
  /^\#===edge===/  {  # base vectors for unit cell e[] in units of b[]
    if( NF<10 ) { d=2 } else { d=3 }
    for( i=0; i<d; i++ ) for( j=0; j<d; j++ ) {
       e[i*3+j]=0;
       for( k=0; k<d; k++ ) e[i*3+j] += $(2+i*d+k) * b[k*3+j];
    }
    print "edge-vector ",e[0],e[1],e[3],e[4] > "/dev/stderr"
  }
  /^\#===uvec===/  { # polyline2=1 thickness3=3 color4=4 forward_arrow10=1 depth=10
      s1="2 1 0 3 4 4 10 0 -1 0.000 0 0 -1 1 0 2\n\t 0 0 3 240 330\n\t %d %d %d %d\n"
      if ( NF < 10 ) { d=2 } else { d=3 }
      u1x=b[0]*$(2  )+b[3]*$(3  ); u1y=b[1]*$(2  )+b[4]*$(3  );
      u2x=b[0]*$(2+d)+b[3]*$(3+d); u2y=b[1]*$(2+d)+b[4]*$(3+d);
      printf s1,(ox-.5)*sc,(oy+.5)*sc,(u1x-.5+ox)*sc,(u1y+.5+oy)*sc;
      printf s1,(ox-.5)*sc,(oy+.5)*sc,(u2x-.5+ox)*sc,(u2y+.5+oy)*sc;
      outc( (u1x-.5),(u1y+.5),0,"b1",4);
      outc( (u2x-.5),(u2y+.5),0,"b2",4); }
  /^\#===pbcf===/  {  # frame around unitcell (parallel epiped) 4*2
      printf "2 3 0 1 2 2 0 0 -1 0.000 0 0 -1 0 0 %d\n\t",(NF-1)/d+1;
      c1x=c1y=0; # new centers (odd + even) for compact figure
      for ( i=0; i<NF-1; i+=d ) {
         x=b[0]*$(i+2)+b[3]*$(i+3);
         y=b[1]*$(i+2)+b[4]*$(i+3);
         c1x+=x;  c1y+=y;
         printf " %d %d",(x+ox)*sc,(y+oy)*sc; } 
      x=b[0]*$(2)+b[3]*$(3);
      y=b[1]*$(2)+b[4]*$(3);
      printf " %d %d\n",(x+ox)*sc,(y+oy)*sc;
      # for nonRectangles we could use shortest distance to two centers
      c1x/=int((NF-1)/d); c1y/=int((NF-1)/d);
      # print int((NF-1)/d) > "/dev/stderr"
      printf "1 3 0 1 2 7 4 0 20 0.00 1 0.00 %d %d %d %d %d %d %d %d\n",\
        (c1x+ox)*sc,(c1y+oy)*sc,50,50,\
        (c1x+ox)*sc,(c1y+oy)*sc,(c1x+ox)*sc+50,(c1y+oy)*sc;
  }
  (nr == 1) { vn=$1;vnw=$2; printf "# NN=%d Nw=%d\n",vn,vnw }
  (nr > 1) && (nr < vn+2) { # scan sites
       ax[$1]=(b[0]*$2+b[3]*$3); # $1 must be 0...vn-1
       ay[$1]=(b[1]*$2+b[4]*$3);
  }
  (nr > vn+1) { # scan bonds
       ldd[$1]=0;                  # no dash, dash, dotted, dash+dotted
       ldl[$1]=0;                  # dash-length
       if (match($0,"j2")) { ldd[$1]=1; ldl[$1]=12; } # dashed
       if (match($0,"j3")) { ldd[$1]=1; ldl[$1]=6; } # dashed
       if (match($0,"j4")) { ldd[$1]=1; ldl[$1]=3; } # dashed
       ls1[$1]=$2;  # site 1
       ls2[$1]=$3;  # site 2
       lbc[$1]=0; if (match($0,"+")) lbc[$1]=1;  # PBC
  }
  END {
     if (center>0) {
       print "try to build compact figure" > "/dev/stderr";
       for (i=0; i<vn; i++) { m1=0;m2=0;
         # move near to center of UC
         # distance measured in real (-c) or UC coordinates (-C)
         if (center==1) {
           dist2=(ax[i]-c1x)^2\
                +(ay[i]-c1y)^2;
         } else {
           # x_E = x_B*E^1 = x_B*E^A/det(E) # inverse transformation to e-Coordinates
           dist2=((( ax[i]*e[4]-ay[i]*e[3])/(e[0]*e[4]-e[1]*e[3])-0.5)^2\
                + ((-ax[i]*e[1]+ay[i]*e[0])/(e[0]*e[4]-e[1]*e[3])-0.5)^2);
         }
         for( x1=-1; x1<2; x1++ ) for( x2=-1; x2<2; x2++ ) {
          tx=ax[i]+x1*e[0]+x2*e[3]; # moved test vector
          ty=ay[i]+x1*e[1]+x2*e[4];
          if (center==1) {
           # check center
           distt = (tx-c1x)^2 + (ty-c1y)^2;
          } else {
           distt=((( tx*e[4]-ty*e[3])/(e[0]*e[4]-e[1]*e[3])-0.5)^2\
                + ((-tx*e[1]+ty*e[0])/(e[0]*e[4]-e[1]*e[3])-0.5)^2);
          }
          if (distt < dist2) { m1=x1;m2=x2;dist2=distt; }
         }
         ax[i]=ax[i]+m1*e[0]+m2*e[3];
         ay[i]=ay[i]+m1*e[1]+m2*e[4];
       }
     }
     for (i=0; i<vn; i++) 
       outs(ax[i],ay[i],0,i,0);
     for (i=0; i<vnw; i++) {
#   pline=2 sub={1=polyline} style={0,1=dashed} width=2 pcolor=-1 fcolor=-1\
#          depth=0 unused=0 afill=-1 dashlen=0.000 join=0 cap=0 radius=-1\
#          farrow=0 barrow=0 npoints=2\n\t x1 y1 x2 y2\n
       s1="2 1 %d %d -1 0 5 0 -1 %4.2f 0 0 -1 0 0 2\n\t%d %d %d %d\n";
       if (lbc[i] || center)   # PBC
       { m1=0; m2=0;
         dist2=(ax[ls1[i]]-ax[ls2[i]])^2\
              +(ay[ls1[i]]-ay[ls2[i]])^2;
         for( x1=-2; x1<3; x1++ ) for( x2=-2; x2<3; x2++ ) if( x1 || x2 ) # if( (x1!=0) || (x2!=0) )
         if ((ax[ls1[i]]+x1*e[0]+x2*e[3]-ax[ls2[i]])^2\
            +(ay[ls1[i]]+x1*e[1]+x2*e[4]-ay[ls2[i]])^2 < dist2)
         {  m1=x1; m2=x2; 
            dist2=(ax[ls1[i]]+m1*e[0]+m2*e[3]-ax[ls2[i]])^2\
                 +(ay[ls1[i]]+m1*e[1]+m2*e[4]-ay[ls2[i]])^2;
         }
         if (m1 || m2) {
           printf s1,ldd[i],1,ldl[i],(ax[ls1[i]]+m1*e[0]+m2*e[3]+ox)*sc,(ay[ls1[i]]+m1*e[1]+m2*e[4]+oy)*sc,(ax[ls2[i]]+ox)*sc,(ay[ls2[i]]+oy)*sc;
           printf s1,ldd[i],1,ldl[i],(ax[ls2[i]]-m1*e[0]-m2*e[3]+ox)*sc,(ay[ls2[i]]-m1*e[1]-m2*e[4]+oy)*sc,(ax[ls1[i]]+ox)*sc,(ay[ls1[i]]+oy)*sc;
           outs( ax[ls1[i]]+m1*e[0]+m2*e[3], ay[ls1[i]]+m1*e[1]+m2*e[4], 0, ls1[i], 1-center );
           outs( ax[ls2[i]]-m1*e[0]-m2*e[3], ay[ls2[i]]-m1*e[1]-m2*e[4], 0, ls2[i], 1-center );
         } else
         printf s1,ldd[i],2,ldl[i],(ax[ls1[i]]+ox)*sc,(ay[ls1[i]]+oy)*sc,(ax[ls2[i]]+ox)*sc,(ay[ls2[i]]+oy)*sc;
       } else {
         printf s1,ldd[i],2,ldl[i],(ax[ls1[i]]+ox)*sc,(ay[ls1[i]]+oy)*sc,(ax[ls2[i]]+ox)*sc,(ay[ls2[i]]+oy)*sc;
       }
     }
  }
  ' $file
exit

